@@ -20,24 +20,24 @@ module Liquid
       case markup
       when NamedSyntax
         @variables = variables_from_string($2)
-        @name = $1
+        @name = Expression.parse($1)
       when SimpleSyntax
         @variables = variables_from_string(markup)
-        @name = "'#{@variables.to_s}'"
+        @name = @variables.to_s
       else
         raise SyntaxError.new(options[:locale].t("errors.syntax.cycle".freeze))
       end
     end
 
     def render(context)
-      context.registers[:cycle] ||= Hash.new(0)
+      context.registers[:cycle] ||= {}
 
       context.stack do
-        key = context[@name]
-        iteration = context.registers[:cycle][key]
-        result = context[@variables[iteration]]
+        key = context.evaluate(@name)
+        iteration = context.registers[:cycle][key].to_i
+        result = context.evaluate(@variables[iteration])
         iteration += 1
-        iteration  = 0  if iteration >= @variables.size
+        iteration  = 0 if iteration >= @variables.size
         context.registers[:cycle][key] = iteration
         result
       end
@@ -48,7 +48,7 @@ module Liquid
     def variables_from_string(markup)
       markup.split(',').collect do |var|
         var =~ /\s*(#{QuotedFragment})\s*/o
-        $1 ? $1 : nil
+        $1 ? Expression.parse($1) : nil
       end.compact
     end
   end
